# set version string here
# if run from svn checkout, the svn revision will be picked up instead

__version__ = '1.1.9'
    
from os import path, curdir
import sys
import types
import numpy as np
from numbers import Number

from appdata import configdir
from symbols import pom_globals

from misc import frozen_base, source_base, darwin_base

if hasattr(sys, 'frozen') and sys.frozen == "windows_exe":
    revpath = path.join(frozen_base, 'SVNREVISION')
elif hasattr(sys, 'frozen') and sys.platform == 'darwin':
    revpath = path.join(darwin_base, 'SVNREVISION')
else:
    revpath = path.join(source_base, 'SVNREVISION')

def svn_revision():
    ''' tries to determine svn version by reading the .svn/all-wcprops file'''
    svnpath = path.join(path.dirname(__file__),'.svn','all-wcprops')
    if path.exists(svnpath):
        #print 'found svn'
        rev = 0
        for l in open(svnpath):
            if l[0] == '/':
                rev = max(int(l.split('/')[5]), rev)
        return rev
    elif hasattr(sys, "frozen") or path.exists(revpath): 
        try:
            rev = int(open(revpath).read().strip())
        except IOError:
            print 'IOError'
            return None
        else:
            return rev
    return None

rev = svn_revision()
if rev is not None:
    __version__ = '000%d'%rev


def load_peaks():
    pkg = __import__('peak_o_mat',fromlist=['peaks'])
    mod = getattr(pkg, 'peaks')
    for name in dir(mod):
        sym = getattr(mod, name)
        if type(sym) == types.FunctionType:
            pom_globals.update({name: sym})

def load_userfunc():
    sys.path.append(configdir())
    try:
        mod = __import__('userfunc',fromlist=[])
    except ImportError:
        print 'no userfunc.py'
    else:
        for name in dir(mod):
            sym = getattr(mod, name)
            if type(sym) in [types.FunctionType,np.ufunc] or isinstance(sym, Number):
                pom_globals.update({name: sym})

load_userfunc()
load_peaks()
