
import wx
from wx import stc
from stc import PythonSTC
from wx.lib.pubsub import pub as Publisher

import code
from StringIO import StringIO
import sys
import re

import images

class Locals(dict):
    autocall = []
    def __getitem__(self, name):
        #if type(dict.__getitem__(self, name)) == func:
        #    return dict.__getitem__(self, name)()
        if name in self.autocall:
            return dict.__getitem__(self, name)()
        else:
            return dict.__getitem__(self, name)

    def add(self, name, val, autocall=False):
        self[name] = val
        if autocall:
            self.autocall.append(name)

    def __setitem__(self, name, val):
        if name in self.autocall:
            raise Exception, 'overwriting \'%s\' not allowed'%name
        else:
            dict.__setitem__(self,name,val)

class Interpreter(code.InteractiveInterpreter):
    def __init__(self, controller):
        self.controller = controller
        self.errline = None

        code.InteractiveInterpreter.__init__(self, self.init_locals())

        self.out = StringIO()

    def init_locals(self):
        locs = Locals(locals())
        locs.add('add_plot', self.controller.add_plot)
        locs.add('add_set', self.controller.add_set)
        locs.add('project', self.controller.project)
        def _get_model():
            return self.controller.fit_controller.model
        locs.add('model', _get_model, True)
        def _update_view():
            self.controller.update_tree()
            self.controller.update_plot()
        locs.add('sync', _update_view)
        def _get_active_set():
            return self.controller.active_set
        locs.add('aset', _get_active_set, True)
        #locs.add('intro', intro, False)
        return locs

    def write(self, text):
        self.out.write(text)
        mat = re.match(r'.+, line (\d+)\D+', text)
        if mat is not None:
            self.errline = int(mat.groups()[0])-1

    def getresult(self):
        ret = self.out.getvalue(), self.errline
        self.out = StringIO()
        return ret

class CodeController(object):
    def __init__(self, main, view):
        self.controller = main
        self.view = view
        self.interpreter = Interpreter(self.controller)
        Interactor().Install(self, view)

    def run(self, source):
        tmp = sys.stdout
        sys.stdout = self.interpreter
        self.interpreter.runsource(source, '<buffer>', 'exec')
        sys.stdout = tmp
        return self.interpreter.getresult()

    def _get_code(self):
        return self.view.editor.GetText()
    def _set_code(self, text):
        self.view.silent = True
        self.view.editor.SetValue(text)
        self.view.silent = False
    code = property(_get_code, _set_code)

class Interactor:
    def Install(self, controller, view):
        self.controller = controller
        self.view = view

        self.view.Bind(wx.EVT_BUTTON, self.OnExecute)
        self.view.Bind(wx.EVT_CLOSE, self.OnClose)
        self.view.Bind(stc.EVT_STC_CHANGE, self.OnEditCode)

        self.view.Bind(wx.EVT_IDLE, self.OnIdle)

    def OnIdle(self, evt):
        self.view.btn_runselectedcode.Enable(len(self.view.editor.GetSelectedText().strip())>0)

    def OnEditCode(self, evt):
        Publisher.sendMessage(('code','changed'))

    def OnExecute(self, evt):
        wx.BeginBusyCursor()
        if evt.GetEventObject() == self.view.btn_runcode:
            val, errline = self.controller.run(self.view.editor.GetText())
        else:
            anc, pos = self.view.editor.GetSelection()
            start = self.view.editor.LineFromPosition(anc)
            val, errline = self.controller.run(self.view.editor.GetSelectedText())
            if errline is not None:
                errline += start

        self.view.txt_result.AppendText(val+'\n')

        if errline is not None:
            self.view.editor.SelectLine(errline)
        wx.EndBusyCursor()

    def OnClose(self, evt):
        self.controller.controller.show_codeeditor(False)

class CodeEditorFrame(wx.Frame):
    def __init__(self, parent):
        style = wx.DEFAULT_FRAME_STYLE|wx.FRAME_FLOAT_ON_PARENT
        wx.Frame.__init__(self, parent, style=style)

        self.setup_controls()
        self.layout()

        self.SetTitle('Code Editor')

        ico = wx.EmptyIcon()
        ico.CopyFromBitmap(images.get_bmp('logosmall.png'))
        self.pom_ico = ico
        self.SetIcon(ico)

    def Show(self, state):
        if not hasattr(self, '_centered'):
            self._centered = True
            self.CenterOnParent()
        wx.Frame.Show(self,state)

    def _set_silent(self, state):
        self.SetEvtHandlerEnabled(not state)
    silent = property(fset=_set_silent)

    def setup_controls(self):
        self.panel = wx.Panel(self)
        self.editor = CodeEditor(self.panel)
        self.btn_runcode = wx.Button(self.panel, label='Run')
        self.btn_runselectedcode = wx.Button(self.panel, label='Run Selection')
        self.txt_result = wx.TextCtrl(self.panel, size=(500,120), style=wx.TE_MULTILINE)

    def layout(self):
        vbox = wx.BoxSizer(wx.VERTICAL)
        vbox.Add(self.editor, 1, wx.EXPAND|wx.ALL, 2)

        hbox = wx.BoxSizer(wx.HORIZONTAL)
        hbox.Add(wx.Window(self.panel, size=(1,1)), 1)
        hbox.Add(self.btn_runselectedcode, 0, wx.RIGHT, 5)
        hbox.Add(self.btn_runcode, 0)

        vbox.Add(hbox, 0, wx.EXPAND|wx.RIGHT|wx.LEFT|wx.BOTTOM, 2)
        vbox.Add(self.txt_result, 0, wx.EXPAND|wx.ALL, 2)
        vbox.Add(wx.Window(self.panel, size=(10,10)))
        self.panel.SetSizer(vbox)

class CodeEditor(PythonSTC):
    def __init__(self, parent, style=wx.BORDER_NONE):
        PythonSTC.__init__(self, parent, -1, style=style)
        self.SetUpEditor()

    # Some methods to make it compatible with how the wxTextCtrl is used
    def SetValue(self, value):
        if wx.USE_UNICODE:
            value = value.decode('iso8859_1')
        val = self.GetReadOnly()
        self.SetReadOnly(False)
        self.SetText(value)
        self.EmptyUndoBuffer()
        self.SetSavePoint()
        self.SetReadOnly(val)

    def SetEditable(self, val):
        self.SetReadOnly(not val)

    def IsModified(self):
        return self.GetModify()

    def Clear(self):
        self.ClearAll()

    def SetInsertionPoint(self, pos):
        self.SetCurrentPos(pos)
        self.SetAnchor(pos)

    def ShowPosition(self, pos):
        line = self.LineFromPosition(pos)
        #self.EnsureVisible(line)
        self.GotoLine(line)

    def GetLastPosition(self):
        return self.GetLength()

    def GetPositionFromLine(self, line):
        return self.PositionFromLine(line)

    def GetRange(self, start, end):
        return self.GetTextRange(start, end)

    def GetSelection(self):
        return self.GetAnchor(), self.GetCurrentPos()

    def SetSelection(self, start, end):
        self.SetSelectionStart(start)
        self.SetSelectionEnd(end)

    def SelectLine(self, line):
        start = self.PositionFromLine(line)
        end = self.GetLineEndPosition(line)
        self.SetSelection(start, end)

    def SetUpEditor(self):
        """
        This method carries out the work of setting up the demo editor.            
        It's seperate so as not to clutter up the init code.
        """
        import keyword

        self.SetModEventMask(wx.stc.STC_MOD_INSERTTEXT|wx.stc.STC_PERFORMED_USER)

        self.SetLexer(stc.STC_LEX_PYTHON)
        self.SetKeyWords(0, " ".join(keyword.kwlist))

        # Enable folding
        self.SetProperty("fold", "1" )

        # Highlight tab/space mixing (shouldn't be any)
        self.SetProperty("tab.timmy.whinge.level", "1")

        # Set left and right margins
        self.SetMargins(2,2)

        # Set up the numbers in the margin for margin #1
        self.SetMarginType(1, wx.stc.STC_MARGIN_NUMBER)
        # Reasonable value for, say, 4-5 digits using a mono font (40 pix)
        self.SetMarginWidth(1, 40)

        # Indentation and tab stuff
        self.SetIndent(4)               # Proscribed indent size for wx
        self.SetIndentationGuides(True) # Show indent guides
        self.SetBackSpaceUnIndents(True)# Backspace unindents rather than delete 1 space
        self.SetTabIndents(True)        # Tab key indents
        self.SetTabWidth(4)             # Proscribed tab size for wx
        self.SetUseTabs(False)          # Use spaces rather than tabs, or
        # TabTimmy will complain!
        # White space
        self.SetViewWhiteSpace(False)   # Don't view white space

        # EOL: Since we are loading/saving ourselves, and the
        # strings will always have \n's in them, set the STC to
        # edit them that way.            
        self.SetEOLMode(wx.stc.STC_EOL_LF)
        self.SetViewEOL(False)

        # No right-edge mode indicator
        self.SetEdgeMode(stc.STC_EDGE_NONE)

        # Setup a margin to hold fold markers
        self.SetMarginType(2, stc.STC_MARGIN_SYMBOL)
        self.SetMarginMask(2, stc.STC_MASK_FOLDERS)
        self.SetMarginSensitive(2, True)
        self.SetMarginWidth(2, 12)

        # and now set up the fold markers
        self.MarkerDefine(stc.STC_MARKNUM_FOLDEREND,     stc.STC_MARK_BOXPLUSCONNECTED,  "white", "black")
        self.MarkerDefine(stc.STC_MARKNUM_FOLDEROPENMID, stc.STC_MARK_BOXMINUSCONNECTED, "white", "black")
        self.MarkerDefine(stc.STC_MARKNUM_FOLDERMIDTAIL, stc.STC_MARK_TCORNER,  "white", "black")
        self.MarkerDefine(stc.STC_MARKNUM_FOLDERTAIL,    stc.STC_MARK_LCORNER,  "white", "black")
        self.MarkerDefine(stc.STC_MARKNUM_FOLDERSUB,     stc.STC_MARK_VLINE,    "white", "black")
        self.MarkerDefine(stc.STC_MARKNUM_FOLDER,        stc.STC_MARK_BOXPLUS,  "white", "black")
        self.MarkerDefine(stc.STC_MARKNUM_FOLDEROPEN,    stc.STC_MARK_BOXMINUS, "white", "black")

        # Global default style
        if wx.Platform == '__WXMSW__':
            self.StyleSetSpec(stc.STC_STYLE_DEFAULT,
                'fore:#000000,back:#FFFFFF,face:Courier New')
        elif wx.Platform == '__WXMAC__':
            # TODO: if this looks fine on Linux too, remove the Mac-specific case 
            # and use this whenever OS != MSW.
            self.StyleSetSpec(stc.STC_STYLE_DEFAULT,
                'fore:#000000,back:#FFFFFF,face:Monaco')
        else:
            defsize = wx.SystemSettings.GetFont(wx.SYS_ANSI_FIXED_FONT).GetPointSize()
            self.StyleSetSpec(stc.STC_STYLE_DEFAULT,
                'fore:#000000,back:#FFFFFF,face:Courier,size:%d'%defsize)

        # Clear styles and revert to default.
        self.StyleClearAll()

        # Following style specs only indicate differences from default.
        # The rest remains unchanged.

        # Line numbers in margin
        self.StyleSetSpec(wx.stc.STC_STYLE_LINENUMBER,'fore:#000000,back:#99A9C2')
        # Highlighted brace
        self.StyleSetSpec(wx.stc.STC_STYLE_BRACELIGHT,'fore:#00009D,back:#FFFF00')
        # Unmatched brace
        self.StyleSetSpec(wx.stc.STC_STYLE_BRACEBAD,'fore:#00009D,back:#FF0000')
        # Indentation guide
        self.StyleSetSpec(wx.stc.STC_STYLE_INDENTGUIDE, "fore:#CDCDCD")

        # Python styles
        self.StyleSetSpec(wx.stc.STC_P_DEFAULT, 'fore:#000000')
        # Comments
        self.StyleSetSpec(wx.stc.STC_P_COMMENTLINE,  'fore:#008000,back:#F0FFF0')
        self.StyleSetSpec(wx.stc.STC_P_COMMENTBLOCK, 'fore:#008000,back:#F0FFF0')
        # Numbers
        self.StyleSetSpec(wx.stc.STC_P_NUMBER, 'fore:#008080')
        # Strings and characters
        self.StyleSetSpec(wx.stc.STC_P_STRING, 'fore:#800080')
        self.StyleSetSpec(wx.stc.STC_P_CHARACTER, 'fore:#800080')
        # Keywords
        self.StyleSetSpec(wx.stc.STC_P_WORD, 'fore:#000080,bold')
        # Triple quotes
        self.StyleSetSpec(wx.stc.STC_P_TRIPLE, 'fore:#800080,back:#FFFFEA')
        self.StyleSetSpec(wx.stc.STC_P_TRIPLEDOUBLE, 'fore:#800080,back:#FFFFEA')
        # Class names
        self.StyleSetSpec(wx.stc.STC_P_CLASSNAME, 'fore:#0000FF,bold')
        # Function names
        self.StyleSetSpec(wx.stc.STC_P_DEFNAME, 'fore:#008080,bold')
        # Operators
        self.StyleSetSpec(wx.stc.STC_P_OPERATOR, 'fore:#800000,bold')
        # Identifiers. I leave this as not bold because everything seems
        # to be an identifier if it doesn't match the above criterae
        self.StyleSetSpec(wx.stc.STC_P_IDENTIFIER, 'fore:#000000')

        # Caret color
        self.SetCaretForeground("BLUE")
        # Selection background
        self.SetSelBackground(1, '#66CCFF')

        self.SetSelBackground(True, wx.SystemSettings_GetColour(wx.SYS_COLOUR_HIGHLIGHT))
        self.SetSelForeground(True, wx.SystemSettings_GetColour(wx.SYS_COLOUR_HIGHLIGHTTEXT))

    def RegisterModifiedEvent(self, eventHandler):
        self.Bind(wx.stc.EVT_STC_CHANGE, eventHandler)

def new(controller, parent):
    return CodeController(controller, CodeEditorFrame(parent))

if __name__ == '__main__':
    app = wx.PySimpleApp(None)
    c = new(None, None)
    app.MainLoop()
